pub enum Error {
ActionNotInSchema(String),
CompileError(CompileError),
EncodeError(EncodeError),
SolverError(SolverError),
SolverUnknown,
PolicyNotWellTyped {
errs: Vec<ValidationError>,
},
DecodeModel(DecodeError),
ConcretizeError(ConcretizeError),
ModelInvalid {
assert: Term,
},
NoPolicies,
}Expand description
Top-level errors from the whole cedar-policy-symcc crate.
Variants
ActionNotInSchema(String)
Action not found in schema.
CompileError(CompileError)
Errors during symbolic compilation.
EncodeError(EncodeError)
Errors from the SMT encoder.
SolverError(SolverError)
Solver-related errors.
SolverUnknown
Solver returned unknown.
PolicyNotWellTyped
Policy is not well-typed.
Fields

errs: Vec<ValidationError>Errors from the policy validator.
DecodeModel(DecodeError)
Failed to decode the SMT model.
ConcretizeError(ConcretizeError)
Errors during concretization.
ModelInvalid
Model successfully decoded and concretized, but the resulting model is invalid.
NoPolicies
Empty-list of policies was passed to a function that expects at least one policy