pub enum SolverError {
Io(Error),
Solver(String),
UnrecognizedSolverOutput(String),
SolverMarkedFailed,
}Expand description
Errors when interacting with a Solver instance.
Corresponds to various errors in the Lean version at Cedar.SymCC.Solver
Variants
Io(Error)
IO error.
Solver(String)
Error from the solver.
UnrecognizedSolverOutput(String)
Unrecognized solver output.
SolverMarkedFailed
The solver was marked as failed and can no longer be used.