pub enum DecodeError {
Show 17 variants
SExprParse(SExprParseError),
ParseIntError(ParseIntError),
IntegerOverflow,
UnexpectedModel,
UnknownType(SExpr),
UnknownLiteral(SExpr),
UnmatchedType(TermType, TermType),
UnmatchedFieldType(TermType, TermType),
InvalidSetType(SExpr),
InvalidOptionType(SExpr),
SetUnionNonLiterals(Term, Term),
UnmatchedRecordType,
UnknownVariable(String),
UnknownUUF(String),
UnexpectedUnaryFunctionForm(SExpr),
BitVecError(BitVecError),
ZeroWidthBitVec,
}Expand description
Errors during decoding, i.e., converting SMT terms
to our internal Term representation.
Variants
SExprParse(SExprParseError)
Error parsing an s-expression
ParseIntError(ParseIntError)
Failed to parse an SMT numeral.
IntegerOverflow
Integer overflow.
UnexpectedModel
Model of an unexpected form returned by the solver.
UnknownType(SExpr)
Unknown SMT type.
UnknownLiteral(SExpr)
Unknown SMT literal.
UnmatchedType(TermType, TermType)
Unmatched types.
UnmatchedFieldType(TermType, TermType)
Unmatched field type.
InvalidSetType(SExpr)
Invalid set type.
InvalidOptionType(SExpr)
Invalid option type.
SetUnionNonLiterals(Term, Term)
set.union applied to non-literals.
UnmatchedRecordType
Unmatched record type fields.
UnknownVariable(String)
Unknown variable.
UnknownUUF(String)
Unknown unary function.
UnexpectedUnaryFunctionForm(SExpr)
Unexpected form of unary function model.
BitVecError(BitVecError)
Bit-vector error.
ZeroWidthBitVec
Bitvector of a zero width, which we do not support.