CompiledPolicySet

Struct CompiledPolicySet 

pub struct CompiledPolicySet { /* private fields */ }
Represents a symbolically compiled policyset. This can be fed into various functions on CedarSymCompiler for efficient solver queries (that don’t have to repeat symbolic compilation).

Implementations

Source

impl CompiledPolicySet

pub fn compile( pset: &PolicySet, env: &RequestEnv, schema: &Schema, ) -> Result<Self>

Compile a policyset for the given RequestEnv.

This does all the validating and well-typing that you need; you need not (and should not) call WellTypedPolicies::from_policies() prior to calling this.

pub fn compile_with_custom_symenv( pset: &PolicySet, env: &RequestEnv, schema: &Schema, symenv: SymEnv, ) -> Result<Self>

Compile a set of policies for the given RequestEnv, using a custom SymEnv rather than the one that would naturally be derived from this RequestEnv.

Most often, you want compile() instead. compile_with_custom_symenv() is generally used to compile with SymEnvs that are more concrete than the default SymEnvs, with constraints/concretizations of the entity hierarchy or request variables.

Caller is responsible for some currently-undocumented invariants about the relationship between the RequestEnv and the SymEnv. This function has no analogue in the Lean (as of this writing). Use at your own risk.